@@ -1,41 +1,44 @@
 """ The task of this module, is to save, load and init new games """
+
 from __future__ import print_function
 
 import os
-
 from gi.repository import Gtk
 from gi.repository import GObject
 
-from pychess import Savers
+
 from pychess.Savers.ChessFile import LoadingError
-from pychess.Savers import *
+from pychess.Savers import chessalpha2, epd, fen, pgn, png
 from pychess.System import conf
 from pychess.System.Log import log
 from pychess.System.protoopen import isWriteable
-from pychess.System.uistuff import GladeWidgets, keep
-from pychess.Utils.const import *
+from pychess.System.uistuff import GladeWidgets
+from pychess.Utils.const import UNFINISHED_STATES, ABORTED, ABORTED_AGREEMENT, LOCAL, ARTIFICIAL
 from pychess.Utils.Offer import Offer
 from pychess.widgets import gamenanny, gamewidget
 
 
-def generalStart (gamemodel, player0tup, player1tup, loaddata=None):
+def generalStart(gamemodel, player0tup, player1tup, loaddata=None):
     """ The player tuples are:
-        (The type af player in a System.const value,
-         A callable creating the player,
-         A list of arguments for the callable,
-         A preliminary name for the player)
+    (The type af player in a System.const value,
+    A callable creating the player,
+    A list of arguments for the callable,
+    A preliminary name for the player)
 
-        If loaddata is specified, it should be a tuple of:
-        (A text uri or fileobj,
-         A Savers.something module with a load function capable of loading it,
-         An int of the game in file you want to load,
-         The position from where to start the game) """
-    log.debug("ionest.generalStart: %s\n %s\n %s" % (gamemodel, player0tup, player1tup))
+    If loaddata is specified, it should be a tuple of:
+    (A text uri or fileobj,
+    A Savers.something module with a load function capable of loading it,
+    An int of the game in file you want to load,
+    The position from where to start the game)
+    """
+
+    log.debug("ionest.generalStart: %s\n %s\n %s" %
+              (gamemodel, player0tup, player1tup))
 
     gmwidg = gamewidget.GameWidget(gamemodel)
     gmwidg.connect("game_close_clicked", closeGame, gamemodel)
 
-    #worker.publish((gmwidg,gamemodel))
+    # worker.publish((gmwidg,gamemodel))
     gamewidget.attachGameWidget(gmwidg)
     gamenanny.nurseGame(gmwidg, gamemodel)
     log.debug("ionest.generalStart: -> emit gmwidg_created: %s" % (gmwidg))
@@ -48,7 +51,7 @@ def generalStart (gamemodel, player0tup, player1tup, loaddata=None):
         type, func, args, prename = playertup
         if type != LOCAL:
             players.append(func(*args))
-            #if type == ARTIFICIAL:
+            # if type == ARTIFICIAL:
             #    def readyformoves (player, color):
             #        gmwidg.setTabText(gmwidg.display_text))
             #    players[i].connect("readyForMoves", readyformoves, i)
@@ -57,7 +60,7 @@ def generalStart (gamemodel, player0tup, player1tup, loaddata=None):
             # issue tracker, we need to give human players special treatment
             player = func(gmwidg, *args)
             players.append(player)
-            
+
             # Connect to conf
             if i == 0 or (i == 1 and player0tup[0] != LOCAL):
                 key = "firstName"
@@ -66,27 +69,34 @@ def generalStart (gamemodel, player0tup, player1tup, loaddata=None):
                 key = "secondName"
                 alt = _("Guest")
             if prename == conf.get(key, alt):
-                conf.notify_add(key, lambda *a:player.setName(conf.get(key,alt)))
+                conf.notify_add(key,
+                                lambda *a: player.setName(conf.get(key, alt)))
 
     if player0tup[0] == ARTIFICIAL and player1tup[0] == ARTIFICIAL:
-        def emit_action (action, param):
+
+        def emit_action(action, param):
             if gmwidg.isInFront():
                 gamemodel.curplayer.emit("offer", Offer(action, param=param))
-        gmwidg.board.connect("action", lambda b,action,param: emit_action(action, param))
 
-    log.debug("ionest.generalStart: -> gamemodel.setPlayers(): %s" % (gamemodel))
+        gmwidg.board.connect(
+            "action", lambda b, action, param: emit_action(action, param))
+
+    log.debug("ionest.generalStart: -> gamemodel.setPlayers(): %s" %
+              (gamemodel))
     gamemodel.setPlayers(players)
-    log.debug("ionest.generalStart: <- gamemodel.setPlayers(): %s" % (gamemodel))
-    
+    log.debug("ionest.generalStart: <- gamemodel.setPlayers(): %s" %
+              (gamemodel))
+
     # Starting
     if loaddata:
         try:
             uri, loader, gameno, position = loaddata
-            gamemodel.loadAndStart (uri, loader, gameno, position)
+            gamemodel.loadAndStart(uri, loader, gameno, position)
             if position != gamemodel.ply:
                 gmwidg.board.view.shown = position
         except LoadingError as e:
-            d = Gtk.MessageDialog (type=Gtk.MessageType.WARNING, buttons=Gtk.ButtonsType.OK)
+            d = Gtk.MessageDialog(type=Gtk.MessageType.WARNING,
+                                  buttons=Gtk.ButtonsType.OK)
             d.set_markup(_("<big><b>Error loading game</big></b>"))
             d.format_secondary_text(", ".join(str(a) for a in e.args))
             d.show()
@@ -96,12 +106,14 @@ def generalStart (gamemodel, player0tup, player1tup, loaddata=None):
         if gamemodel.variant.need_initial_board:
             for player in gamemodel.players:
                 player.setOptionInitialBoard(gamemodel)
-        log.debug("ionest..generalStart: -> gamemodel.start(): %s" % (gamemodel))
+        log.debug("ionest..generalStart: -> gamemodel.start(): %s" %
+                  (gamemodel))
         gamemodel.start()
-        log.debug("ionest.generalStart: <- gamemodel.start(): %s" % (gamemodel))
-    
-    log.debug("ionest.generalStart: returning gmwidg=%s\n gamemodel=%s" % \
-        (gmwidg, gamemodel))
+        log.debug("ionest.generalStart: <- gamemodel.start(): %s" %
+                  (gamemodel))
+
+    log.debug("ionest.generalStart: returning gmwidg=%s\n gamemodel=%s" %
+              (gmwidg, gamemodel))
     return gmwidg, gamemodel
 
 ################################################################################
@@ -113,18 +125,24 @@ savedialog = None
 enddir = {}
 saveformats = None
 exportformats = None
+
+savers = (chessalpha2, epd, fen, pgn, png)
+for saver in savers:
+    enddir[saver.__ending__] = saver
+
+
 def getOpenAndSaveDialogs():
     global opendialog, savedialog, enddir, savecombo, savers, saveformats, exportformats
 
     if not opendialog:
-        savers = [getattr(Savers, s) for s in Savers.__all__]
-        for saver in savers:
-            enddir[saver.__ending__] = saver
-
-        opendialog = Gtk.FileChooserDialog(_("Open Game"), None, Gtk.FileChooserAction.OPEN,
-            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN, Gtk.ResponseType.ACCEPT))
-        savedialog = Gtk.FileChooserDialog("", None, Gtk.FileChooserAction.SAVE,
-            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_SAVE, Gtk.ResponseType.ACCEPT))
+        opendialog = Gtk.FileChooserDialog(
+            _("Open Game"), None, Gtk.FileChooserAction.OPEN,
+            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_OPEN,
+             Gtk.ResponseType.ACCEPT))
+        savedialog = Gtk.FileChooserDialog(
+            "", None, Gtk.FileChooserAction.SAVE,
+            (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, Gtk.STOCK_SAVE,
+             Gtk.ResponseType.ACCEPT))
         savedialog.set_current_folder(os.path.expanduser("~"))
         saveformats = Gtk.ListStore(str, str, GObject.TYPE_PYOBJECT)
         exportformats = Gtk.ListStore(str, str, GObject.TYPE_PYOBJECT)
@@ -150,10 +168,10 @@ def getOpenAndSaveDialogs():
             label, ending = saver.__label__, saver.__ending__
             endstr = "(%s)" % ending
             f = Gtk.FileFilter()
-            f.set_name(label+" "+endstr)
+            f.set_name(label + " " + endstr)
             if hasattr(enddir[ending], "load"):
-                f.add_pattern("*."+ending)
-                all.add_pattern("*."+ending)
+                f.add_pattern("*." + ending)
+                all.add_pattern("*." + ending)
                 opendialog.add_filter(f)
                 saveformats.append([label, endstr, saver])
                 i += 1
@@ -164,6 +182,7 @@ def getOpenAndSaveDialogs():
 
         # Add widgets to the savedialog
         savecombo = Gtk.ComboBox()
+        savecombo.set_name("savecombo")
         crt = Gtk.CellRendererText()
         savecombo.pack_start(crt, True)
         savecombo.add_attribute(crt, 'text', 0)
@@ -179,21 +198,25 @@ def getOpenAndSaveDialogs():
 # Saving                                                                       #
 ################################################################################
 
-def saveGame (game):
+
+def saveGame(game, position=None):
     if not game.isChanged():
         return
-    if game.uri and isWriteable (game.uri):
-        saveGameSimple (game.uri, game)
+    if game.uri and isWriteable(game.uri):
+        saveGameSimple(game.uri, game, position=position)
     else:
-        return saveGameAs (game)
+        return saveGameAs(game, position=position)
 
-def saveGameSimple (uri, game):
+
+def saveGameSimple(uri, game, position=None):
     ending = os.path.splitext(uri)[1]
-    if not ending: return
+    if not ending:
+        return
     saver = enddir[ending[1:]]
-    game.save(uri, saver, append=False)
+    game.save(uri, saver, append=False, position=position)
 
-def saveGamePGN (game):
+
+def saveGamePGN(game):
     if conf.get("saveOwnGames", False) and not game.hasLocalPlayer():
         return True
     filename = conf.get("autoSaveFormat", "pychess")
@@ -209,13 +232,14 @@ def saveGamePGN (game):
     try:
         game.save(uri, saver, append)
         return True
-    except IOError as e:
+    except IOError:
         return False
 
-def saveGameAs (game, position=None):
+
+def saveGameAs(game, position=None, export=False):
     opendialog, savedialog, enddir, savecombo, savers = getOpenAndSaveDialogs()
 
-    if position is not None:
+    if export:
         savecombo.set_model(exportformats)
     else:
         savecombo.set_model(saveformats)
@@ -225,8 +249,8 @@ def saveGameAs (game, position=None):
     title = _("Save Game") if position is None else _("Export position")
     savedialog.set_title(title)
     while True:
-        savedialog.set_current_name("%s %s %s" %
-                                   (game.players[0], _("vs."), game.players[1]))
+        savedialog.set_current_name(
+            "%s %s %s" % (game.players[0], _("vs."), game.players[1]))
 
         res = savedialog.run()
         if res != Gtk.ResponseType.ACCEPT:
@@ -234,18 +258,20 @@ def saveGameAs (game, position=None):
 
         uri = savedialog.get_filename()
         ending = os.path.splitext(uri)[1]
-        if ending.startswith("."): ending = ending[1:]
+        if ending.startswith("."):
+            ending = ending[1:]
         append = False
 
         if savecombo.get_active() == 0:
-            if not ending in enddir:
-                d = Gtk.MessageDialog(
-                        type=Gtk.MessageType.ERROR, buttons=Gtk.ButtonsType.OK)
+            if ending not in enddir:
+                d = Gtk.MessageDialog(type=Gtk.MessageType.ERROR,
+                                      buttons=Gtk.ButtonsType.OK)
                 folder, file = os.path.split(uri)
-                d.set_markup(
-                          _("<big><b>Unknown file type '%s'</b></big>") % ending)
-                d.format_secondary_text(_("Was unable to save '%(uri)s' as PyChess doesn't know the format '%(ending)s'.") % {
-                                            'uri': uri, 'ending': ending})
+                d.set_markup(_("<big><b>Unknown file type '%s'</b></big>") %
+                             ending)
+                d.format_secondary_text(_(
+                    "Was unable to save '%(uri)s' as PyChess doesn't know the format '%(ending)s'.") %
+                    {'uri': uri, 'ending': ending})
                 d.run()
                 d.hide()
                 continue
@@ -253,16 +279,17 @@ def saveGameAs (game, position=None):
                 saver = enddir[ending]
         else:
             index = savecombo.get_active()
-            format = saveformats[index] if position is None else exportformats[index]
+            format = exportformats[index] if export else saveformats[index]
             saver = format[2]
-            if not ending in enddir or not saver == enddir[ending]:
+            if ending not in enddir or not saver == enddir[ending]:
                 uri += ".%s" % saver.__ending__
 
-        if os.path.isfile(uri) and not os.access (uri, os.W_OK):
-            d = Gtk.MessageDialog(type=Gtk.MessageType.ERROR, buttons=Gtk.ButtonsType.OK)
+        if os.path.isfile(uri) and not os.access(uri, os.W_OK):
+            d = Gtk.MessageDialog(type=Gtk.MessageType.ERROR,
+                                  buttons=Gtk.ButtonsType.OK)
             d.set_markup(_("<big><b>Unable to save file '%s'</b></big>") % uri)
-            d.format_secondary_text(
-                _("You don't have the necessary rights to save the file.\n\
+            d.format_secondary_text(_(
+                "You don't have the necessary rights to save the file.\n\
 Please ensure that you have given the right path and try again."))
             d.run()
             d.hide()
@@ -270,14 +297,16 @@ Please ensure that you have given the right path and try again."))
 
         if os.path.isfile(uri):
             d = Gtk.MessageDialog(type=Gtk.MessageType.QUESTION)
-            d.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL, _("_Replace"),
-                        Gtk.ResponseType.ACCEPT)
+            d.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                          _("_Replace"), Gtk.ResponseType.ACCEPT)
             if saver.__append__:
                 d.add_buttons(Gtk.STOCK_ADD, 1)
             d.set_title(_("File exists"))
             folder, file = os.path.split(uri)
-            d.set_markup(_("<big><b>A file named '%s' already exists. Would you like to replace it?</b></big>") % file)
-            d.format_secondary_text(_("The file already exists in '%s'. If you replace it, its content will be overwritten.") % folder)
+            d.set_markup(_(
+                "<big><b>A file named '%s' already exists. Would you like to replace it?</b></big>") % file)
+            d.format_secondary_text(_(
+                "The file already exists in '%s'. If you replace it, its content will be overwritten.") % folder)
             replaceRes = d.run()
             d.hide()
 
@@ -287,15 +316,17 @@ Please ensure that you have given the right path and try again."))
                 continue
         else:
             print(repr(uri))
-            
+
         try:
             game.save(uri, saver, append, position)
         except IOError as e:
             d = Gtk.MessageDialog(type=Gtk.MessageType.ERROR)
             d.add_buttons(Gtk.STOCK_OK, Gtk.ResponseType.OK)
             d.set_title(_("Could not save the file"))
-            d.set_markup(_("<big><b>PyChess was not able to save the game</b></big>"))
-            d.format_secondary_text(_("The error was: %s") % ", ".join(str(a) for a in e.args))
+            d.set_markup(_(
+                "<big><b>PyChess was not able to save the game</b></big>"))
+            d.format_secondary_text(_("The error was: %s") % ", ".join(
+                str(a) for a in e.args))
             d.run()
             d.hide()
             continue
@@ -305,26 +336,26 @@ Please ensure that you have given the right path and try again."))
     savedialog.hide()
     return res
 
+
 ################################################################################
 # Closing                                                                      #
 ################################################################################
-def closeAllGames (pairs):
+def closeAllGames(pairs):
     log.debug("ionest.closeAllGames")
     response = None
-    changedPairs = [(gmwidg, game) for gmwidg, game in pairs if game.isChanged()]
+    changedPairs = [(gmwidg, game) for gmwidg, game in pairs
+                    if game.isChanged()]
     if len(changedPairs) == 0:
         response = Gtk.ResponseType.OK
 
     elif len(changedPairs) == 1:
         response = closeGame(*changedPairs[0])
     else:
-        markup = "<big><b>" + \
-                ngettext("There is %d game with unsaved moves.",
-                "There are %d games with unsaved moves.",
-                len(changedPairs)) % len(changedPairs) + \
-                " " + _("Save moves before closing?") + \
-                "</b></big>"
-                               
+        markup = "<big><b>" + ngettext("There is %d game with unsaved moves.",
+                                       "There are %d games with unsaved moves.",
+                                       len(changedPairs)) % len(changedPairs) + " " + \
+            _("Save moves before closing?") + "</b></big>"
+
         if conf.get("autoSave", False):
             for gmwidg, game in changedPairs:
                 x = saveGamePGN(game)
@@ -332,11 +363,10 @@ def closeAllGames (pairs):
                     response = Gtk.ResponseType.OK
                 else:
                     response = None
-                    markup = "<b><big>" + \
-                            _("Unable to save to configured file. Save the games before closing?") + \
-                            "</big></b>"
+                    markup = "<b><big>" + _("Unable to save to configured file. \
+                                            Save the games before closing?") + "</big></b>"
                     break
-                    res
+#                    res
         if response is None:
             widgets = GladeWidgets("saveGamesDialog.glade")
             dialog = widgets["saveGamesDialog"]
@@ -351,17 +381,20 @@ def closeAllGames (pairs):
             renderer = Gtk.CellRendererToggle()
             renderer.props.activatable = True
             treeview.append_column(Gtk.TreeViewColumn("", renderer, active=0))
-            treeview.append_column(Gtk.TreeViewColumn("", Gtk.CellRendererText(), text=1))
+            treeview.append_column(Gtk.TreeViewColumn("",
+                                                      Gtk.CellRendererText(),
+                                                      text=1))
             for gmwidg, game in changedPairs:
-                liststore.append((True, "%s %s %s" %
-                                 (game.players[0], _("vs."), game.players[1])))
+                liststore.append((True, "%s %s %s" % (game.players[0], _("vs."), game.players[1])))
 
-            def callback (cell, path):
+            def callback(cell, path):
                 if path:
                     liststore[path][0] = not liststore[path][0]
                 saves = len(tuple(row for row in liststore if row[0]))
-                saveLabel.set_text(ngettext("_Save %d document", "_Save %d documents", saves) % saves)
+                saveLabel.set_text(ngettext(
+                    "_Save %d document", "_Save %d documents", saves) % saves)
                 saveLabel.set_use_underline(True)
+
             renderer.connect("toggled", callback)
 
             callback(None, None)
@@ -369,17 +402,17 @@ def closeAllGames (pairs):
             while True:
                 response = dialog.run()
                 if response == Gtk.ResponseType.YES:
-                    for i in range(len(liststore)-1, -1, -1):
+                    for i in range(len(liststore) - 1, -1, -1):
                         checked, name = liststore[i]
                         if checked:
-                            gmwidg, game = changedPairs[i]
-                            if saveGame(game) == Gtk.ResponseType.ACCEPT:
-                                liststore.remove(liststore.get_iter((i,)))
+                            cgmwidg, cgame = changedPairs[i]
+                            if saveGame(cgame) == Gtk.ResponseType.ACCEPT:
+                                liststore.remove(liststore.get_iter((i, )))
                                 del changedPairs[i]
-                                if game.status in UNFINISHED_STATES:
-                                    game.end(ABORTED, ABORTED_AGREEMENT)
-                                game.terminate()
-                                gamewidget.delGameWidget(gmwidg)
+                                if cgame.status in UNFINISHED_STATES:
+                                    cgame.end(ABORTED, ABORTED_AGREEMENT)
+                                cgame.terminate()
+                                gamewidget.delGameWidget(cgmwidg)
                             else:
                                 break
                     else:
@@ -388,7 +421,8 @@ def closeAllGames (pairs):
                     break
             dialog.destroy()
 
-    if response not in (Gtk.ResponseType.DELETE_EVENT, Gtk.ResponseType.CANCEL):
+    if response not in (Gtk.ResponseType.DELETE_EVENT,
+                        Gtk.ResponseType.CANCEL):
         pairs = [(gmwidg, game) for gmwidg, game in pairs]
         for gmwidg, game in pairs:
             if game.status in UNFINISHED_STATES:
@@ -399,37 +433,36 @@ def closeAllGames (pairs):
 
     return response
 
-def closeGame (gmwidg, game):
+
+def closeGame(gmwidg, game):
     log.debug("ionest.closeGame")
     response = None
     if not game.isChanged():
         response = Gtk.ResponseType.OK
     else:
-        markup = "<b><big>" + \
-                _("Save the current game before you close it?") + \
-                "</big></b>"
+        markup = "<b><big>" + _("Save the current game before you close it?") + "</big></b>"
         if conf.get("autoSave", False):
             x = saveGamePGN(game)
             if x:
                 response = Gtk.ResponseType.OK
             else:
-                markup = "<b><big>" + \
-                        _("Unable to save to configured file. Save the current game before you close it?") + \
-                        "</big></b>"
+                markup = "<b><big>" + _("Unable to save to configured file. \
+                                        Save the current game before you close it?") + "</big></b>"
         if response is None:
-            d = Gtk.MessageDialog (type = Gtk.MessageType.WARNING)
+            d = Gtk.MessageDialog(type=Gtk.MessageType.WARNING)
             d.add_button(_("Close _without Saving"), Gtk.ResponseType.OK)
             d.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL)
             if game.uri:
                 d.add_button(Gtk.STOCK_SAVE, Gtk.ResponseType.YES)
-            else: d.add_button(Gtk.STOCK_SAVE_AS, Gtk.ResponseType.YES)
+            else:
+                d.add_button(Gtk.STOCK_SAVE_AS, Gtk.ResponseType.YES)
 
             gmwidg.bringToFront()
 
             d.set_markup(markup)
-            d.format_secondary_text (_(
+            d.format_secondary_text(_(
                 "It is not possible later to continue the game,\nif you don't save it."))
-            
+
             response = d.run()
             d.destroy()
 
@@ -438,11 +471,12 @@ def closeGame (gmwidg, game):
             if saveGame(game) != Gtk.ResponseType.ACCEPT:
                 response = Gtk.ResponseType.CANCEL
 
-    if response not in (Gtk.ResponseType.DELETE_EVENT, Gtk.ResponseType.CANCEL):
+    if response not in (Gtk.ResponseType.DELETE_EVENT,
+                        Gtk.ResponseType.CANCEL):
         if game.status in UNFINISHED_STATES:
             game.end(ABORTED, ABORTED_AGREEMENT)
         game.terminate()
-        gamewidget.delGameWidget (gmwidg)
+        gamewidget.delGameWidget(gmwidg)
 
     return response
 
@@ -451,16 +485,18 @@ def closeGame (gmwidg, game):
 ################################################################################
 
 
-class Handler (GObject.GObject):
+class Handler(GObject.GObject):
     """ The goal of this class, is to provide signal handling for the ionest
         module.
         Emit objects are gmwidg, gameobject """
 
     __gsignals__ = {
-        'gmwidg_created': (GObject.SignalFlags.RUN_FIRST, None, (object, object))
+        'gmwidg_created': (GObject.SignalFlags.RUN_FIRST, None,
+                           (object, object))
     }
 
-    def __init__ (self):
+    def __init__(self):
         GObject.GObject.__init__(self)
 
+
 handler = Handler()
